--[[
 * ReaScript Name: Import markers and regions from tab-delimited CSV file
 * Description: See title.
 * Instructions: Select a track. Run.
 * Author: X-Raym
 * Author URI: http://www.extremraym.com
 * Repository: GitHub > X-Raym > EEL Scripts for Cockos REAPER
 * Repository URI: https://github.com/X-Raym/REAPER-EEL-Scripts
 * Links
    Forum Thread https://forum.cockos.com/showthread.php?p=1670961
 * Licence: GPL v3
 * REAPER: 5.0
 * Version: 1.0
--]]

--[[
 * Changelog:
 * v1.0 (2019-01-26)
  + Initial Release
--]]

-- USER CONFIG AREA -----------------------------------------------------------
-- Duplicate and Rename the script if you want to modify this.
-- Else, a script update will erase your mods.

-- Display a message in the console for debugging
function Msg(value)
  if console then
    reaper.ShowConsoleMsg(tostring(value) .. "\n")
  end
end
console = true -- true/false: display debug messages in the console

local info = debug.getinfo(1,'S');
script_path = info.source:match[[^@?(.*[\/])[^\/]-$]] 
script_path2 = script_path .."/sonic/win32"

--(1)pattern,(2)type,(3)A verse B chorus,(4)weight,(5)mask,(6)duration,(7)bar
--(1)shot,(2)0 shot 1 hold,(3)bar,(4)ticks,(5)duration ticks,weight,volume

col_pos = 1 -- Position column index in the CSV
col_pos_end = 4 -- Length column index in the CS
col_len = 6 -- Length column index in the CSV
col_name = 2 -- Name column index in the CSV
col_color = 3
col_pattern = 7
col_ticks = 5
sep = "," -- for csv   sep = "\t" -- for txt
end_script = 0

bpm, beat_per_measure = reaper.GetProjectTimeSignature2(0)
snap_choice = 0
if reaper.MB( "Snap Chord changes to beat ?", "Snap Chords", 4) == 6 then snap_choice = 1 end



------------------------------------------------------- END OF USER CONFIG AREA

function ColorHexToInt(hex)
  hex = hex:gsub("#", "")
  local R = tonumber("0x"..hex:sub(1,2))
  local G = tonumber("0x"..hex:sub(3,4))
  local B = tonumber("0x"..hex:sub(5,6))
  return reaper.ColorToNative(R, G, B)
end

-- Optimization
local reaper = reaper

-- CSV to Table
-- http://lua-users.org/wiki/LuaCsv
function ParseCSVLine (line,sep)
  --[[
  local line = string.gsub(line, "{.*}", "") -- remove { } and text between them
  local line = string.gsub(line, ";.*", "") -- remove ; and text after it
  local line = string.gsub(line, "pattern,PreFill.*", "") -- remove pattern,PreFill line
  
  --]]
  local res = {}
  local pos = 1 
  sep = sep or ','
  while true do
    local c = string.sub(line,pos,pos)
    if (c == "") then break end
    if (c == '"') then
      -- quoted value (ignore separator within)
      local txt = ""
      repeat
        local startp,endp = string.find(line,'^%b""',pos)
        txt = txt..string.sub(line,startp+1,endp-1)
        pos = endp + 1
        c = string.sub(line,pos,pos)
        if (c == '"') then txt = txt..'"' end
        -- check first char AFTER quoted string, if it is another
        -- quoted string without separator, then append it
        -- this is the way to "escape" the quote char in a quote. example:
        --   value1,"blub""blip""boing",value3  will result in blub"blip"boing  for the middle
      until (c ~= '"')
      table.insert(res,txt)
      assert(c == sep or c == "")
      pos = pos + 3
    else
      -- no quotes used, just look for the first separator
      local startp,endp = string.find(line,sep,pos)
      if (startp) then
        table.insert(res,string.sub(line,pos,startp-1))
        pos = endp + 1
      else
        -- no separator found -> use rest of string and terminate
        table.insert(res,string.sub(line,pos))
        break
      end
    end
  end
  return res
end


-- UTILITIES -------------------------------------------------------------


function ReverseTable(t)
    local reversedTable = {}
    local itemCount = #t
    for k, v in ipairs(t) do
        reversedTable[itemCount + 1 - k] = v
    end
    return reversedTable
end

--------------------------------------------------------- END OF UTILITIES

function read_lines(filepath)

  lines = {}

  local f = io.input(filepath)
  repeat

    s = f:read ("*l") -- read one line

    if s then  -- if not end of file (EOF)
      table.insert(lines, ParseCSVLine (s,sep))
    end

  until not s  -- until end of file
         

  f:close()

end

function snap_all_regions_to_grid()

      reaper.Main_OnCommand(40754, 0) -- Enable snap
      
      region_count , num_markersOut, num_regionsOut = reaper.CountProjectMarkers(0)

      for i=0, region_count -1 do
      
       --EnumProjectMarkers(i, is_region, region_start, region_end, #name, region_id)
       retval, isrgnOut, posOut, rgnendOut, region_name, markrgnindexnumberOut, colorOut = reaper.EnumProjectMarkers3(0, i)      
      
      --if isrgnOut then
         
         region_snapped_start =  reaper.SnapToGrid(0, posOut)
         region_snapped_end =  reaper.SnapToGrid(0, rgnendOut) 
         
         --SetProjectMarker(region_id, 1, region_snapped_start, region_snapped_end, #name)
         
         reaper.SetProjectMarker3( 0, markrgnindexnumberOut, isrgnOut, region_snapped_start, region_snapped_end, region_name , colorOut )
         
      end
    --end  
end

local clock = os.clock
function sleep(n)  -- seconds
  local t0 = clock()
  while clock() - t0 <= n do end
end

function get_chords() 
    
  --Msg("Get Chords")   
  
  -- https://code.soundsoftware.ac.uk/projects/sonic-annotator/files
  -- https://code.soundsoftware.ac.uk/projects/nnls-chroma/files
  -- https://code.soundsoftware.ac.uk/projects/qm-vamp-plugins/files
  
  os_type = reaper.GetOS()  --  "Win32", "Win64", "OSX32", "OSX64", or "Other".
  
  if os_type == "Win32" or os_type == "Win64" then
    
    sel_item = reaper.GetSelectedMediaItem( 0, 0 )
    
    item_pos =  reaper.GetMediaItemInfo_Value( sel_item, "D_POSITION" )
    
    if not sel_item then 
    
      reaper.MB( "Select one audio item", "Select Audio Item", 0 ) 
      
      end_script = 1 
      
    end
    
    if sel_item then 
    
      item_take = reaper.GetMediaItemTake( sel_item, 0 )
      
      source = reaper.GetMediaItemTake_Source( item_take )
      
      filename = reaper.GetMediaSourceFileName(source, "")
      --Msg("filename ".. filename)
      
      filename_no_ext = string.gsub(filename, '[.]%w%w%w$','')
      
      take_type = reaper.TakeIsMIDI( item_take )
      
      if take_type then
    
        reaper.MB( "Item is MIDI", "Select Audio Item", 0 ) 
        
        end_script = 1
        
      end 
    end  
    
      if not take_type and end_script == 0 then 
      
       reaper.ExecProcess([[cmd.exe /C " cd ]] .. script_path .. [[sonic/win32/ & set VAMP_PATH=]] .. script_path .. [[sonic/win32/vamp; ]].. script_path .. [[ & "sonic-annotator.exe" -d vamp:nnls-chroma:chordino:simplechord -w csv "]].. filename ..[["]],0)
       
       filetxt = filename_no_ext .. "_vamp_nnls-chroma_chordino_simplechord.csv"
        
      end
  
  end
  
  
  
  if os_type ~= "Win32" and os_type ~= "Win64" then
    
    --Msg("Mac OS")
    sel_item = reaper.GetSelectedMediaItem( 0, 0 )
    
    if not sel_item then 
    
      reaper.MB( "Select one audio item", "Select Audio Item", 0 ) 
      
      end_script = 1 
      
    end
    
    if sel_item then 
    
      item_take = reaper.GetMediaItemTake( sel_item, 0 )
      
      source = reaper.GetMediaItemTake_Source( item_take )
      
      filename = reaper.GetMediaSourceFileName(source, "")
      --Msg("filename ".. filename)
      
      filename_no_ext = string.gsub(filename, '[.]%w%w%w$','')
      
      take_type = reaper.TakeIsMIDI( item_take )
      
      if take_type then
    
        reaper.MB( "Item is MIDI", "Select Audio Item", 0 ) 
        
        end_script = 1
        
      end 
    end  
    
      if not take_type and end_script == 0 then 
      
        os.execute('export VAMP_PATH="'..script_path..'sonic/mac32/vamp"; cd "'..script_path..'sonic/mac64/"; ./sonic-annotator -d vamp:nnls-chroma:chordino:simplechord -w csv "'..filename..'"')
                         
                  
        filetxt = filename_no_ext .. "_vamp_nnls-chroma_chordino_simplechord.csv"
        
      end
  
  end  
  

 end

-- Main function
function main()

  bpm1, beat_per_measure = reaper.GetProjectTimeSignature2(0)

  folder = filetxt:match[[^@?(.*[\/])[^\/]-$]]

  for i, line in ipairs( lines ) do
    --if line[col_pattern] == "pattern" then 
   
     if i > 1 then
      
      -- Name Variables
      local pos = tonumber(line[col_pos])
      --local pos_end = tonumber(line[col_pos] +1) --tonumber(line[col_pos_end])
      local len =  line[col_len] 
      local name = line[col_name]
      local color = 0
      
      --local bpm2 = string.gsub(bpm, " bpm", "")
      
      --Msg("bpm2 ".. bpm2)

        color = ColorHexToInt("#3776EB")|0x1000000
        
        --Msg(" Marker " .. pos .."Name ".. name)
        if name == "N" then i = i +1
           else
               reaper.AddProjectMarker2(0, false, pos, 0, name, -1, color)
               --reaper.AddTempoTimeSigMarker( 0, pos, bpm2, 0, 0, 0 )
        end
      end
         
         
         --snap_all_regions_to_grid()
                  
    end
      
end




-- INIT

--retval, filetxt = reaper.GetUserFileNameForRead("", "Import Chordino chords to regions", input_choose)

--if retval then

  reaper.PreventUIRefresh(1)

  reaper.Undo_BeginBlock() -- Begining of the undo block. Leave it at the top of your main function.
  
  --reaper.ClearConsole()

  get_chords()

  if end_script == 0 then  
    
    read_lines(filetxt)
  
    main()
    
    if snap_choice == 1 then
      snap_all_regions_to_grid()
    end
    
    commandID1 = reaper.NamedCommandLookup("_SWSMARKERLIST13")
    reaper.Main_OnCommand(commandID1, 0) -- SWS: Convert markers to regions
    
  end
  
  reaper.Undo_EndBlock("ReaTrak get chords from item to regions", -1) -- End of the undo block. Leave it at the bottom of your main function.

  reaper.UpdateArrange()

  reaper.PreventUIRefresh(-1)
  
  
      



